package be.iminds.ilabt.jfed.espec;

import be.iminds.ilabt.jfed.espec.model.*;
import be.iminds.ilabt.jfed.espec.parser.ESpecConstants;
import be.iminds.ilabt.jfed.espec.parser.ESpecDefaults;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

/**
 * ESpecLogic
 */
public class ESpecLogic {
    private ESpecLogic() { throw new RuntimeException("static helper class"); }

    public static boolean hasAnsibleNode(@Nonnull ExperimentSpecification espec) {
        return espec.getAnsible() != null &&
                espec.getAnsible().getAnsibleHostSpec().getHostType() != AnsibleHostSpec.HostType.LOCAL;
    }

    public static boolean isAnsibleSubPart(@Nonnull UploadLikeSpec uploadLikeSpec,
                                           @Nonnull ExperimentSpecification espec) {
        if (espec.getAnsible() == null) return false;
        if (uploadLikeSpec instanceof AnsiblePlaybookSpec) return true;
        if (uploadLikeSpec instanceof AnsibleGalaxySpec) return true;
        if (uploadLikeSpec instanceof AnsiblePlaybookSpec.ExtraVarsJsonFileSpec) return true;

        List<UploadLikeSpec> ansibleUploads = new ArrayList<>();
        ansibleUploads.addAll(espec.getAnsible().getAnsibleHostSpec().getUploadLists());
        ansibleUploads.addAll(espec.getAnsible().getAnsibleHostSpec().getExecuteLists());
        ansibleUploads.addAll(espec.getAnsible().getAnsibleGalaxySpecs());
        ansibleUploads.addAll(espec.getAnsible().getAnsiblePlaybookSpecs());

        for (UploadLikeSpec ansibleUploadLikeSpec : ansibleUploads) {
            //this EXPLICITLY needs to use real equality, just to be certain.
            //(and yes, this loop is not efficient, but that shouldn't be a problem here)
            //noinspection ObjectEquality
            if (ansibleUploadLikeSpec == uploadLikeSpec) {
                return true;
            }
        }
        return false;
    }

    public static boolean isAnsibleNode(@Nonnull String clientId,
                                        @Nonnull ExperimentSpecification espec) {
        if (espec.getAnsible() == null) return false;
        if (espec.getAnsible().getAnsibleHostSpec().getHostType() == AnsibleHostSpec.HostType.LOCAL) return false;

        String ansibleNodeName = espec.getAnsible().getAnsibleHostSpec().getNodeName() == null ?
                ESpecDefaults.ANSIBLE_NODE_NAME :
                espec.getAnsible().getAnsibleHostSpec().getNodeName();
        return clientId.equals(ansibleNodeName);
    }

    /**
     * This assumes there IS an ansible section in the ExperimentSpecification
     * @param experimentSpecification
     * @return
     */
    @Nonnull
    public static String getAnsibleNodeName(@Nonnull ExperimentSpecification experimentSpecification) {
        if (experimentSpecification.getAnsible() == null)
            throw new IllegalStateException("Ansible section in ExperimentSpecification was assumed but not found");
        String ansibleNodeName = experimentSpecification.getAnsible().getAnsibleHostSpec().getNodeName();
        if (ansibleNodeName == null) return ESpecDefaults.ANSIBLE_NODE_NAME;
        return ansibleNodeName;
    }

    public static boolean mustRunOn(@Nonnull String clientId,
                                    @Nullable Collection<String> ansibleGroupNames,
                                    @Nonnull ExperimentSpecification espec,
                                    @Nonnull DirSpec dirSpec) {
        if (dirSpec.getNodes() == null) {
            if (dirSpec.isAnsibleContent())
                return isAnsibleNode(clientId, espec);
            else
                return espec.getConfig().isIncludeAnsibleNodeInAllNodes() || !isAnsibleNode(clientId, espec);
        } else
            return mustRunOnBecauseOfNodes(dirSpec.getNodes(), clientId, ansibleGroupNames);
    }
    public static boolean mustRunOn(@Nonnull String clientId,
                                    @Nullable Collection<String> ansibleGroupNames,
                                    @Nonnull ExperimentSpecification espec,
                                    @Nonnull UploadLikeSpec aSpec) {
        if (isAnsibleNode(clientId, espec))
            return isAnsibleSubPart(aSpec, espec);
        else {
            if (aSpec.getNodes() != null && !aSpec.getNodes().isEmpty()) {
                //if there is a list of nodes, only run on these nodes, not on others
                return mustRunOnBecauseOfNodes(aSpec.getNodes(), clientId, ansibleGroupNames);
            }

            //run on all normal nodes nodes. Check if also needs to run on ansible node.
            return espec.getConfig().isIncludeAnsibleNodeInAllNodes() || !isAnsibleSubPart(aSpec, espec);
        }
    }

    static boolean mustRunOnBecauseOfNodes(@Nullable Collection<String> nodes,
                                           @Nonnull String clientId,
                                           @Nullable Collection<String> ansibleGroupNames) {
        if (nodes != null) {
            boolean mustRun = nodes.contains(clientId);

            //check for ansible groups
            if (ansibleGroupNames != null && !mustRun && !ansibleGroupNames.isEmpty())
                mustRun = nodes.stream()
                        .filter(s -> s.startsWith("ansible_group:"))
                        .map(s -> s.substring("ansible_group:".length()))
                        .anyMatch(ansibleGroupNames::contains);

            return mustRun;
        }
        return false;
    }

}
